from flask import Flask, request, send_from_directory
from flask_cors import CORS
import utils as utils
from trans import Trans
#from pprint import pprint

config = utils.get_config_dict()
git_cmd = utils.git_cmd(config)

app = Flask(__name__)
CORS(app)
trans = Trans()

@app.route('/version', methods=['GET'])
def version():
    app.logger.debug('hit version')
    ret = utils.return_version(version=git_cmd.describe())
    
    return utils.xml_response(ret)

@app.route('/version_xsd', methods=['GET'])# TODO: unnecessary?  
def version_xsd():
    app.logger.debug('hit version_xsd')
    xsd_str = utils.get_xsd(config)
    version_str = utils.get_xsd_version(xsd_str)
    
    return utils.xml_response(utils.return_version(version_str))

@app.route('/update', methods=['POST'])
def update():
    app.logger.debug('hit update')
    #req = request.get_json()
    git_cmd.pull()
    app.logger.info("pulled {log}".format(log=git_cmd.log("-n 1")))
    ret = utils.return_ok()

    return utils.xml_response(ret)

@app.route('/update_xsd', methods=['POST'])# TODO: unnecessary / problematic? 
def update_xsd():
    app.logger.debug('hit update xsd')
    xsd_str = utils.get_xsd(config)
    xsd_xml = utils.parse(xsd_str)
    if xsd_xml:
        ret = utils.save_xsd(config, xsd_str, xsd_file_name)
    else:
        ret = utils.return_error(error='error on atempt to parse xsd file from server')

    return  utils.xml_response(ret)

@app.route('/validate', methods=['POST'])# TODO: '?v=2.2.0' 
def validate():
    app.logger.debug('hit validate')
    xv = request.args.get('v')
    xml_str = request.data
    xml_tree = utils.parse(xml_str)
    if xml_tree:
        print('=== xml_tree OK ===')
        xsd_str = utils.get_xsd(config, xsd_version=xv, xml_str=xml_str)
        if  xsd_str:
            ret = utils.validate(xml_str=xml_str, xsd_str=xsd_str)
        else:
            ret = utils.return_error(error='xsd content not readable')
    else: 
        print('=== xml_tree ??? ===') 
        ret = utils.return_error(error='invalid xml data')

    return  utils.xml_response(ret)
    
@app.route('/validation.html', methods=['GET'])
def validation():
    app.logger.debug('hit validation.html')
    # pprint(vars(request)) 
    l = str(request.accept_languages).split(',')[0][0:2]
    if l != 'de':
        l = 'en'
    x = "['2.2.0','2.1.1','2.1.0']"
    return trans.show_html(version=git_cmd.describe(), language=l, 
      xsd_versions=x)

@app.route('/js/<fn>', methods=['GET'])
def js_folder(fn):
    app.logger.debug('hit js folder')
    return send_from_directory('static/js', fn)

@app.route('/css/<fn>', methods=['GET'])
def css_folder(fn):
    app.logger.debug('hit css folder')
    return send_from_directory('static/css', fn)

@app.route('/img/<fn>', methods=['GET'])
def logo_folder(fn):
    app.logger.debug('hit logo folder')
    return send_from_directory('static/img', fn)

if __name__ == '__main__':
    app.run(host=config['server']['host'], port=config['server']['port'])
