import json
import git
import requests
from flask import Response
from xml.etree import ElementTree as ET
import xmlschema
import sys
from urllib.parse import urlparse
from pprint import pprint

ns = {"w3":"http://www.w3.org/2001/XMLSchema"}

# https://stackoverflow.com/questions/7160737/python-how-to-validate-a-url-in-python-malformed-or-not
def uri_validator(x):
    try:
        result = urlparse(x)
        return all([result.scheme, result.netloc, result.path])
    except:
        return False

def get_config_dict():
    with open('./config.json') as json_config_file:
        config = json.load(json_config_file)

    return config

def git_cmd(config):
    return git.cmd.Git(config['git']['dir'])

def get_xsd_path_file(config, file_name):
    return "{dir}/{file_name}".format(dir=config['xsd']['dir'], file_name=file_name)

def get_xsd(cfg, xsd_version=None, xml_str = None):
    filename =  cfg['xsd']['filename']
    baseURL = cfg['xsd']['externalBaseURL']
    xsd_str = ''
    url = ''
    if xsd_version:
        if xsd_version == 'latest':
            url = baseURL + filename
        else:
            url = baseURL + 'v' + xsd_version + '/' + filename
    else:
      root = parse(xml_str)
      for item in root.attrib.items():
          x = item[1].split()
          for url in x:
              if url.endswith(filename):
                  break
    if uri_validator(url):
        try:
            r = requests.get(url)
            #pprint(vars(r))
            if r.url.endswith(filename): # no bad redirection
                xsd_str = r.text
        except:
            pass
            
    return xsd_str

def save_xsd(config, xsd_str, file_name):
    path_file = get_xsd_path_file(config, file_name)
    file = open(path_file, "w")
    file.write(xsd_str)  
    
    return return_ok() 

def parse(xml_str):
    try:
        tree = ET.fromstring(xml_str)
    except ET.ParseError:
        tree = None
    return tree

def get_xsd_version(xsd_str):
    """The version is tag is a declaration and so 
    not part of the parse tree
    """
    root = parse(xsd_str)
    if root.tag.endswith("schema"):
        return root.attrib.get("version", "")
    else:
        return ""

def validate(xml_str, xsd_str):
    tree = parse(xml_str)
    schema = xmlschema.XMLSchema(xsd_str)
    try:
        schema.validate(tree)
        return return_ok()
    except Exception as error:  
        # except xmlschema.XMLSchemaValidationError as error:
        return return_error(error)
   
def xml_response(xml_str):
    return Response(xml_str, content_type='text/xml; charset=utf-8')

def return_ok():
    return '<ok/>\n'

def return_error(error):
    return '<error>{error}</error>\n'.format(error=error)

def return_version(version):
    return '<version>{version}</version>\n'.format(version=version)
